#include <gtk/gtk.h>
#include <string.h>
#include <time.h>

#include "file_selection.h"
#include "refresh.h"
#include "international.h"


static GtkWidget *window;

static GtkWidget *spinbutton1;
static GtkWidget *spinbutton2;
static GtkWidget *label1;
static GtkWidget *label5;
static GtkWidget *button1;
static GtkWidget *button2;
static GtkWidget *check_save;

static gchar *filenameatrefresh;
static gint time_step;
static gint cicle = 0;
static gdouble spin1 = 1;
static gdouble spin2 = 1;
static gboolean state_save=TRUE;
static gboolean refresh_started = FALSE;

/* interface */
extern GtkWidget *save_at_refresh1;
extern GtkWidget *save_as2;
extern GtkListStore *command_info_table;	/* element for main table */

guint timeout_id = 0;

/* Main Signals Functions */

static gboolean
autorefresh_handler (gpointer data)
{
	gtk_list_store_clear (command_info_table);
	update_clist ();
	if (state_save)
	{
		time_t t;
		struct tm *tm_ptr;
		gchar date_buffer[16];
	
		time (&t);
    	tm_ptr = localtime (&t);
    	strftime (date_buffer, 16, "%Y%m%d%k%M%S", tm_ptr);
	
		save_output(g_strconcat ((gchar *)data,&date_buffer[0],NULL));
	}
	
	if (!cicle) return TRUE;
	
	cicle--;
	if (!cicle) {
		timeout_id = 0;
		
		if (window)
		{
		
			gtk_widget_set_sensitive (button1,TRUE);
			gtk_widget_set_sensitive (label1,TRUE);
			gtk_widget_set_sensitive (label5,TRUE);
			gtk_widget_set_sensitive (check_save,TRUE);
			gtk_widget_set_sensitive (spinbutton1,TRUE);
			gtk_widget_set_sensitive (spinbutton2,TRUE);
			gtk_widget_set_sensitive (button2,FALSE);
			
		}
		
		gtk_widget_set_sensitive (save_at_refresh1,TRUE);
		gtk_widget_set_sensitive (save_as2,TRUE);
		
		refresh_started = FALSE;
		
		return FALSE;
	}
	
	return TRUE;
}

static void
start_autorefresh(GtkButton *button, gpointer user_data)
{
	
	spin1 = (gdouble) gtk_spin_button_get_value_as_int((GtkSpinButton *)spinbutton1);
	time_step = (gint) spin1 * 1000;
	cicle = gtk_spin_button_get_value_as_int((GtkSpinButton *)spinbutton2);
	spin2 = (gdouble) cicle;
	state_save = gtk_toggle_button_get_active ((GtkToggleButton *)check_save);
	gtk_widget_destroy (GTK_WIDGET(user_data));
	
	gtk_widget_set_sensitive (save_at_refresh1,FALSE);
	gtk_widget_set_sensitive (save_as2,FALSE);
	refresh_started = TRUE;
	
	if (state_save)
		on_save_as_clicked (NULL,GINT_TO_POINTER(4));
	else
		timeout_id = gtk_timeout_add (time_step,autorefresh_handler,NULL);
	
}

void 
start_save_autorefresh (gchar *filename)
{
	if (!strlen(filename))
	{
		gtk_widget_set_sensitive (save_at_refresh1,TRUE);
		gtk_widget_set_sensitive (save_as2,TRUE);
		refresh_started = FALSE;
		
		return;
	}
	
	filenameatrefresh = g_strdup (filename);
	
	timeout_id = gtk_timeout_add (time_step,autorefresh_handler,(gpointer *)filenameatrefresh);
}

static void
stop_autorefresh(void)
{
	if (timeout_id) 
		{
			gtk_timeout_remove (timeout_id);	
			timeout_id = 0;
		}
		
	gtk_widget_set_sensitive (save_at_refresh1,TRUE);
	gtk_widget_set_sensitive (save_as2,TRUE);
	gtk_widget_set_sensitive (button1,TRUE);
	gtk_widget_set_sensitive (label1,TRUE);
	gtk_widget_set_sensitive (label5,TRUE);
	gtk_widget_set_sensitive (check_save,TRUE);
	gtk_widget_set_sensitive (spinbutton1,TRUE);
	gtk_widget_set_sensitive (spinbutton2,TRUE);
	gtk_widget_set_sensitive (button2,FALSE);
		
	refresh_started = FALSE;
}

/* close with ESC or Return auto_refresh Dialog */

static gint 
autoref_callback (GtkWidget *w, GdkEventKey *event, gpointer data)
{
	if ((event->keyval==0xFF1B)) /* ESC */
		gtk_widget_destroy (w);
	
	if ((event->keyval==0xFF0D)) /* RETURN */
		start_autorefresh(NULL,data);
	
	return 0;
}

/* AutoRefresh Window */

void
on_autorefresh_clicked (GtkButton * button, gpointer user_data)
{
  GtkWidget *vbox1;
  GtkWidget *table1;
  GtkObject *spinbutton1_adj;
  GtkObject *spinbutton2_adj;
  GtkWidget *hbuttonbox4;
  GtkWidget *hseparator1;


  filenameatrefresh = "";
	
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), _("AutoRefresh"));
  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (window), 500, -1);
  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window), vbox1);

  table1 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);

  label1 = gtk_label_new (_("Time Step (seconds)"));
  gtk_widget_show (label1);
  gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  label5 = gtk_label_new (_("Cicles Number (0 = endless) "));
  gtk_widget_show (label5);
  gtk_table_attach (GTK_TABLE (table1), label5, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

  spinbutton1_adj = gtk_adjustment_new (spin1, 1, 3.16224e+07, 1, 10, 10);
  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
  gtk_widget_show (spinbutton1);
  gtk_table_attach (GTK_TABLE (table1), spinbutton1, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton1), TRUE);

  spinbutton2_adj = gtk_adjustment_new (spin2, 0, 3.16224e+07, 1, 10, 10);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_widget_show (spinbutton2);
  gtk_table_attach (GTK_TABLE (table1), spinbutton2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton2), TRUE);
  
  check_save = gtk_check_button_new_with_mnemonic (_("Save All Refreshes"));
  gtk_toggle_button_set_active ((GtkToggleButton *)check_save, state_save);
  gtk_widget_show (check_save);
  gtk_box_pack_start (GTK_BOX (vbox1), check_save, TRUE, TRUE, 0);
  
  hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_container_add (GTK_CONTAINER (vbox1), hseparator1);

/* buttons ok and cancel (this is the house - mickey mouse!) */
  hbuttonbox4 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox4);
  gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox4, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (hbuttonbox4), 5);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox4), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox4), 10);

  button1 = gtk_button_new_with_mnemonic (_("Start"));
  gtk_widget_show (button1);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), button1);
  
  button2 = gtk_button_new_with_mnemonic (_("Stop"));
  gtk_widget_show (button2);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), button2);
  
  if (refresh_started)
  {
	  gtk_widget_set_sensitive (button1,FALSE);
	  gtk_widget_set_sensitive (label1,FALSE);
	  gtk_widget_set_sensitive (label5,FALSE);
	  gtk_widget_set_sensitive (check_save,FALSE);
	  gtk_widget_set_sensitive (spinbutton1,FALSE);
	  gtk_widget_set_sensitive (spinbutton2,FALSE);
	  
  }
  else
  {
	  gtk_widget_set_sensitive (button2,FALSE);
  }
  
  gtk_widget_show (window);
  gdk_window_raise (window->window);
  
  g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_object_destroy), NULL);
  
  g_signal_connect (G_OBJECT (button1), "clicked",
 		     G_CALLBACK (start_autorefresh), (gpointer) window);
  
  g_signal_connect (G_OBJECT (button2), "clicked",
 		     G_CALLBACK (stop_autorefresh), NULL);
			 
  g_signal_connect ((gpointer) window, "key_press_event",
                    G_CALLBACK (autoref_callback), (gpointer) window);

}
