#include <gtk/gtk.h>
#include <string.h>

#include "list_int_addr.h"
#include "international.h"

#define MAX_LIST_ELEMENTS		100	/* lsof controls max 100 (-i) options */

/* list items */

enum
{
	NUMBER_ITEM = 0,
	ITEM,
	NUM_COLS
};

GPtrArray *tmp_add_list;
GPtrArray *add_list_tmp;
GPtrArray *add_list;

/* variables */

static GtkWidget *entry1;	/* host */
static GtkWidget *entry2;	/* port */
static GtkWidget *combo1;	/* ipv */
static GtkWidget *combo2;	/* protocol */

static GtkListStore *liststore;
static gboolean ok_pressed = FALSE;

/* Function Free Useless Elements */

void
free_useless_elements (GPtrArray *list_to_control, GPtrArray *list_controller)
{
	int i, j, l;
	node_data *elem;
	gboolean existent = FALSE;
	
	l = list_controller->len;
	i = list_to_control->len -1;
	
	for (; i < 0; i--)
	{	
		
		elem = g_ptr_array_index(list_to_control, i);
		existent = FALSE;
		
		for (j=0; j<l; j++)
		{
			
			if (elem == g_ptr_array_index(list_controller, j))
			{
				existent = TRUE;
				j = l;
			}
		}
			
		if (existent == FALSE)
		{
			g_ptr_array_remove_index (list_to_control, i);
			g_free (elem);
		}
		
	}
}

/* Signal Functions */

static void
selection_changed (GtkTreeSelection * selection, gpointer data)
{
	GtkTreeModel *model;
	GtkTreeIter iter;

	if (gtk_tree_selection_get_selected (selection, &model, &iter))
	{
		GtkTreePath *path;
		gint position;
		node_data *tmp_element;

		path = gtk_tree_model_get_path (model, &iter);
		position = gtk_tree_path_get_indices (path)[0];
		tmp_element = g_ptr_array_index (add_list, position);
		gtk_tree_path_free (path);

		gtk_entry_set_text (GTK_ENTRY (entry1), tmp_element->host);
		gtk_entry_set_text (GTK_ENTRY (entry2), tmp_element->port);
		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo1)->entry), tmp_element->ipv);
		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo2)->entry), tmp_element->protocol);
	}
}				/* _selection_changed */

/* This function is exported in settings.c too */

static void
on_set_list_destroy (GtkButton * button, gpointer user_data)
{

	if (ok_pressed == FALSE)
	{
		if ( add_list->len > 0)
			{
				free_useless_elements (add_list, add_list_tmp);
				g_ptr_array_free (add_list,FALSE);
			}
		
		ok_pressed = TRUE;
		gtk_widget_destroy (GTK_WIDGET (user_data));
	}
	else
		ok_pressed = FALSE;
	
}				/* _on_set_list_destroy */

static void
on_ok_pressed (GtkButton * button, gpointer user_data)
{
	int index=0;
	
	if (add_list_tmp->len > 0)
	{
		free_useless_elements (add_list_tmp, add_list);
		g_ptr_array_free (add_list_tmp,FALSE);
		add_list_tmp = g_ptr_array_new ();
	}
	
	for (; index < add_list->len; index++)
	  g_ptr_array_add (add_list_tmp, g_ptr_array_index(add_list,index));

	g_ptr_array_free (add_list,FALSE);
	

	ok_pressed = TRUE;
	gtk_widget_destroy (GTK_WIDGET (user_data));
}

/* Elements List Controls */

static void
add_list_element (void)
{
	GtkTreeIter iter;

	if (add_list->len < MAX_LIST_ELEMENTS)
	{
		int len = 0;

		/* Create new struct on heap - zero's memory */
		node_data *tmp_node = g_new0 (node_data, 1);

		tmp_node->ipv = g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo1)->entry)));
		tmp_node->protocol = g_strdup (gtk_entry_get_text (GTK_ENTRY (GTK_COMBO (combo2)->entry)));
		tmp_node->option = g_strconcat (tmp_node->ipv, tmp_node->protocol, NULL);

		tmp_node->host = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry1)));
		if (strlen (tmp_node->host))
			tmp_node->option = g_strconcat (tmp_node->option, "@", tmp_node->host, NULL);

		tmp_node->port = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry2)));
		if (strlen (tmp_node->port))
			tmp_node->option = g_strconcat (tmp_node->option, ":", tmp_node->port, NULL);

		len = strlen (tmp_node->option);

		if (len > 0)
		{
			int i;
			gboolean state = FALSE;

			for (i = 0; i < add_list->len; i++)
			{
				node_data *elem = g_ptr_array_index (add_list, i);

				if (!g_ascii_strcasecmp (tmp_node->option, elem->option))
				{
					state = TRUE;
					g_free (tmp_node);
					break;
				}
			}

			if (state == FALSE)
			{
				/* add new pointer in add_list */
				g_ptr_array_add (add_list, (gpointer) tmp_node);

				gtk_entry_set_text (GTK_ENTRY (entry1), "");
				gtk_entry_set_text (GTK_ENTRY (entry2), "");
				gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo1)->entry), "");
				gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo2)->entry), "");

				gtk_list_store_append (GTK_LIST_STORE(liststore), &iter);
gtk_list_store_set (liststore, &iter, NUMBER_ITEM, add_list->len,ITEM, tmp_node->option,-1);
				//modified_list = TRUE;

				return;
			}

		}
		else
			g_free (tmp_node);

	}

}				/* _add_list_element */


static void
modify_list_element (GtkButton * button, gpointer data)
{
	GtkTreeSelection *selection = GTK_TREE_SELECTION (data);
	GtkTreeModel *model;
	GtkTreeIter iter;


	if (gtk_tree_selection_get_selected (selection, &model, &iter))
	{
		GtkTreePath *path;
		gint position = 0;
		gint i;
		node_data *tmp_element;

		/* temp datas */
		gchar *option;
		gchar *ipv;
		gchar *protocol;
		gchar *host;
		gchar *port;

		ipv = g_strdup (gtk_entry_get_text
				(GTK_ENTRY (GTK_COMBO (combo1)->entry)));
		protocol =
			g_strdup (gtk_entry_get_text
				  (GTK_ENTRY (GTK_COMBO (combo2)->entry)));
		option = g_strconcat (ipv, protocol, NULL);

		host = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry1)));
		if (strlen (host))
			option = g_strconcat (option, "@", host, NULL);

		port = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry2)));
		if (strlen (port))
			option = g_strconcat (option, ":", port, NULL);

		if (strlen (option) == 0)
			return;

		for (i = 0; i < add_list->len; i++)
		{
			tmp_element = g_ptr_array_index (add_list, i);

			if (!g_ascii_strcasecmp (tmp_element->option, option))
				return;
		}

		path = gtk_tree_model_get_path (model, &iter);
		position = gtk_tree_path_get_indices (path)[0];
		tmp_element = g_ptr_array_index (add_list, position);
		gtk_tree_path_free (path);

		tmp_element->ipv = g_strdup (ipv);
		tmp_element->protocol = g_strdup (protocol);
		tmp_element->host = g_strdup (host);
		tmp_element->port = g_strdup (port);
		tmp_element->option = g_strdup (option);

		//modified_list = TRUE;
		gtk_list_store_set (liststore, &iter, 1, tmp_element->option, -1);
		
		g_free (option);
		g_free (ipv);
		g_free (protocol);
		g_free (host);
		g_free (port);

	}

}				/* _modify_list_element */


static void
remove_list_element (GtkButton * button, gpointer data)
{
	GtkTreeSelection *selection = GTK_TREE_SELECTION (data);
	GtkTreeIter iter;
	GtkTreeModel *model;


	if (gtk_tree_selection_get_selected (selection, &model, &iter))
	{
		GtkTreePath *path;
		guint position;
		
		path = gtk_tree_model_get_path (model, &iter);

		position = gtk_tree_path_get_indices (path)[0];
		g_ptr_array_remove_index (add_list, position);
		
		/* remove line from table */
		
		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
		gtk_tree_path_free (path);
		
		while (position < add_list->len)
		{
			position++;
			gtk_list_store_set (liststore, &iter, 0, position, -1);
			gtk_tree_model_iter_next (model, &iter);
		}

		gtk_entry_set_text (GTK_ENTRY (entry1), "");
		gtk_entry_set_text (GTK_ENTRY (entry2), "");
		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo1)->entry), "");
		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (combo2)->entry), "");

		//modified_list = TRUE;
	}

}				/* _remove_list_element */

/* Set List Window and key events*/

/* window list internet address callback */
static gint 
wlia_callback (GtkWidget *w, GdkEventKey *event, gpointer data)
{
	/* ESC */
	if ( (event->keyval==0xFF1B) )
		on_set_list_destroy (NULL, data);
	
	/* RETURN */
	/*if ( (event->keyval==0xFF0D) )
		on_ok_pressed (NULL, data);*/
	
	return 0;
}

void
on_set_list_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *window_list_int_addr;
	GtkWidget *vbox3;
	GtkWidget *hbox2;
	GtkWidget *table2;
	GtkWidget *label7;
	GtkWidget *label8;
	GtkWidget *label9;
	GtkWidget *label10;
	GList *combo1_items = NULL;
	GtkWidget *combo_entry1;
	GList *combo2_items = NULL;
	GtkWidget *combo_entry2;
	GtkWidget *scrolledwindow1;
	GtkWidget *hbox1;
	GtkWidget *hbuttonbox4;
	GtkWidget *button6;
	GtkWidget *button8;
	GtkWidget *hbuttonbox5;
	GtkWidget *button7;
	GtkWidget *hseparator1;
	GtkWidget *hbuttonbox3;
	GtkWidget *button4;
	GtkWidget *button5;
	GtkWidget *view;	/* element for list */
	GtkTreeSelection *selection;	/* element for list */
	GtkCellRenderer *cell;	/* element for list */

	GdkScreen *screen;

	window_list_int_addr = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (window_list_int_addr),
			      ( _("List of files for Internet address") ) );
	gtk_window_set_position (GTK_WINDOW (window_list_int_addr), GTK_WIN_POS_CENTER);
	gtk_window_set_modal (GTK_WINDOW (window_list_int_addr), TRUE);
	gtk_window_set_resizable (GTK_WINDOW (window_list_int_addr), TRUE);

	screen = gtk_window_get_screen (GTK_WINDOW (window_list_int_addr));
	gtk_window_resize (GTK_WINDOW (window_list_int_addr),
			   (int) (gdk_screen_get_width (screen) / 2),
			   (int) (gdk_screen_get_height (screen) / 2));

	vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox3);
	gtk_container_add (GTK_CONTAINER (window_list_int_addr), vbox3);

	hbox2 = gtk_hbox_new (TRUE, 0);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox2, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox2), 8);

	table2 = gtk_table_new (4, 2, TRUE);
	gtk_widget_show (table2);
	gtk_box_pack_start (GTK_BOX (hbox2), table2, TRUE, TRUE, 8);

	label7 = gtk_label_new (_("Host Address/Name"));
	gtk_widget_show (label7);
	gtk_table_attach (GTK_TABLE (table2), label7, 0, 1, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label7), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label7), 0, 0.5);

	label8 = gtk_label_new (_("List Services/Ports"));
	gtk_widget_show (label8);
	gtk_table_attach (GTK_TABLE (table2), label8, 0, 1, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label8), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label8), 0, 0.5);

	label9 = gtk_label_new (_("IPV"));
	gtk_widget_show (label9);
	gtk_table_attach (GTK_TABLE (table2), label9, 0, 1, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label9), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label9), 0, 0.5);

	label10 = gtk_label_new (_("Protocol"));
	gtk_widget_show (label10);
	gtk_table_attach (GTK_TABLE (table2), label10, 0, 1, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_label_set_justify (GTK_LABEL (label10), GTK_JUSTIFY_LEFT);
	gtk_misc_set_alignment (GTK_MISC (label10), 0, 0.5);

	entry1 = gtk_entry_new ();
	gtk_widget_show (entry1);
	gtk_table_attach (GTK_TABLE (table2), entry1, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	entry2 = gtk_entry_new ();
	gtk_widget_show (entry2);
	gtk_table_attach (GTK_TABLE (table2), entry2, 1, 2, 1, 2,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	combo1 = gtk_combo_new ();
	gtk_widget_show (combo1);
	gtk_table_attach (GTK_TABLE (table2), combo1, 1, 2, 2, 3,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	combo1_items = g_list_append (combo1_items, (gpointer) "");
	combo1_items = g_list_append (combo1_items, (gpointer) ("4"));
	combo1_items = g_list_append (combo1_items, (gpointer) ("6"));
	gtk_combo_set_popdown_strings (GTK_COMBO (combo1), combo1_items);
	gtk_combo_set_value_in_list (GTK_COMBO (combo1),TRUE,TRUE);
	g_list_free (combo1_items);

	combo_entry1 = GTK_COMBO (combo1)->entry;
	gtk_widget_show (combo_entry1);

	combo2 = gtk_combo_new ();
	gtk_widget_show (combo2);
	gtk_table_attach (GTK_TABLE (table2), combo2, 1, 2, 3, 4,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	combo2_items = g_list_append (combo2_items, (gpointer) "");
	combo2_items = g_list_append (combo2_items, (gpointer) _("TCP"));
	combo2_items = g_list_append (combo2_items, (gpointer) _("UDP"));
	gtk_combo_set_popdown_strings (GTK_COMBO (combo2), combo2_items);
	gtk_combo_set_value_in_list (GTK_COMBO (combo2),TRUE,TRUE);
	g_list_free (combo2_items);

	combo_entry2 = GTK_COMBO (combo2)->entry;
	gtk_widget_show (combo_entry2);

	scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_show (scrolledwindow1);
	gtk_box_pack_start (GTK_BOX (hbox2), scrolledwindow1, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),
					GTK_POLICY_AUTOMATIC,
					GTK_POLICY_AUTOMATIC);

/********* start address table *********/

	view = gtk_tree_view_new ();
	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));

	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
	liststore = gtk_list_store_new (NUM_COLS, G_TYPE_INT, G_TYPE_STRING);

	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), TRUE);	/* white - grey alternate */
	
	gtk_tree_view_set_model (GTK_TREE_VIEW (view),
				 GTK_TREE_MODEL (liststore));

	cell = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view), -1,
						     "   #  ", cell, "text",
						     NUMBER_ITEM, NULL);

	cell = gtk_cell_renderer_text_new ();
	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (view), -1,
						     "ITEM", cell, "text",
						     ITEM, NULL);

	gtk_widget_show (view);
	
	gtk_container_add (GTK_CONTAINER (scrolledwindow1), view);

/***************/

	hbox1 = gtk_hbox_new (TRUE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox3), hbox1, FALSE, TRUE, 0);

	hbuttonbox4 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox4);
	gtk_box_pack_start (GTK_BOX (hbox1), hbuttonbox4, FALSE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox4),
				   GTK_BUTTONBOX_END);

	button8 = gtk_button_new_with_mnemonic (_("Modify"));
	gtk_widget_show (button8);
	gtk_container_add (GTK_CONTAINER (hbuttonbox4), button8);
	GTK_WIDGET_SET_FLAGS (button8, GTK_CAN_DEFAULT);

	button6 = gtk_button_new_with_mnemonic (_("Add"));
	gtk_widget_show (button6);
	gtk_container_add (GTK_CONTAINER (hbuttonbox4), button6);
	GTK_WIDGET_SET_FLAGS (button6, GTK_CAN_DEFAULT);

	hbuttonbox5 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox5);
	gtk_box_pack_start (GTK_BOX (hbox1), hbuttonbox5, TRUE, TRUE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox5),
				   GTK_BUTTONBOX_END);

	button7 = gtk_button_new_with_mnemonic (_("Remove"));
	gtk_widget_show (button7);
	gtk_container_add (GTK_CONTAINER (hbuttonbox5), button7);
	GTK_WIDGET_SET_FLAGS (button7, GTK_CAN_DEFAULT);

	hseparator1 = gtk_hseparator_new ();
	gtk_widget_show (hseparator1);
	gtk_box_pack_start (GTK_BOX (vbox3), hseparator1, FALSE, FALSE, 2);

	hbuttonbox3 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox3);
	gtk_box_pack_start (GTK_BOX (vbox3), hbuttonbox3, FALSE, FALSE, 0);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox3),
				   GTK_BUTTONBOX_END);

	button4 = gtk_button_new_from_stock ("gtk-ok");
	gtk_widget_show (button4);
	gtk_container_add (GTK_CONTAINER (hbuttonbox3), button4);
	GTK_WIDGET_SET_FLAGS (button4, GTK_CAN_DEFAULT);

	button5 = gtk_button_new_from_stock ("gtk-cancel");
	gtk_widget_show (button5);
	gtk_container_add (GTK_CONTAINER (hbuttonbox3), button5);
	GTK_WIDGET_SET_FLAGS (button5, GTK_CAN_DEFAULT);
	
	/* print list */
	
	add_list = g_ptr_array_new ();
	
	if ( add_list_tmp->len > 0 )
	{
		node_data *tmp_element;
		int index = 0;
		GtkTreeIter iter;

		while (index < add_list_tmp->len)
		{	
			tmp_element = g_ptr_array_index(add_list_tmp, index);
			g_ptr_array_add (add_list, tmp_element); /* copy on array */
			gtk_list_store_append (GTK_LIST_STORE (liststore), &iter); /* copy on list_store */
			index++;
			gtk_list_store_set (liststore, &iter, NUMBER_ITEM, index, ITEM, tmp_element->option, -1);
		}
	}	

	gtk_widget_show (window_list_int_addr);
	gdk_window_raise (window_list_int_addr->window);

	/* signals */
	
	g_signal_connect (G_OBJECT (window_list_int_addr), "destroy", G_CALLBACK (on_set_list_destroy), (gpointer) window_list_int_addr);
	g_signal_connect (G_OBJECT (button4), "clicked", G_CALLBACK (on_ok_pressed), (gpointer) window_list_int_addr);
	g_signal_connect (G_OBJECT (button5), "clicked", G_CALLBACK (on_set_list_destroy), (gpointer) window_list_int_addr);
	g_signal_connect (G_OBJECT (button6), "clicked", G_CALLBACK (add_list_element), NULL);
	g_signal_connect (G_OBJECT (button7), "clicked", G_CALLBACK (remove_list_element), (gpointer) selection);
	g_signal_connect (G_OBJECT (button8), "clicked", G_CALLBACK (modify_list_element), (gpointer) selection);
	g_signal_connect (G_OBJECT (selection), "changed", G_CALLBACK (selection_changed), (gpointer) view);
	g_signal_connect (G_OBJECT (window_list_int_addr), "key_press_event", G_CALLBACK (wlia_callback), (gpointer) window_list_int_addr);

}				/* _on_set_list_clicked */
