#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>

#include "xmlconfig.h"
#include "list_int_addr.h"


/* list_int_addr */
extern GPtrArray *tmp_add_list;

xmlDocPtr doc;
xmlXPathContextPtr xpcont;

/* init and destroy functions */
int initReadConfig(char *filename) {
	doc = xmlParseFile(filename);
	if(NULL == doc) {
		cleanup();
		return -1;
	}

	xpcont = xmlXPathNewContext(doc);
	if(NULL == xpcont) {
		cleanup();
		return -1;
	}

	return 0;
}

int initWriteConfig(char *filename) {
	doc = xmlParseFile(filename);
	if(NULL == doc) {
		cleanup();
		return -1;
	}

	xpcont = xmlXPathNewContext(doc);
	if(NULL == xpcont) {
		cleanup();
		return -1;
	}

	return 0;
}

void destroyReadConfig() {
	cleanup();
}

int destroyWriteConfig(char *filename, unsigned int compressMode) {
	if(filename == "" || compressMode > 9 || NULL == doc) {
		return -1;
	}

	xmlSetCompressMode(compressMode);

	if(-1 == xmlSaveFile(filename, doc)) {
		return -1;
	}

	cleanup();
	return 0;
}


/* get and set functions */
/* view options */
void getViewOptions(unsigned int vopt[]) {
	vopt[PID] = getNodeIntVal("/configuration/viewoptions/pid/@selected");
	vopt[PGID] = getNodeIntVal("/configuration/viewoptions/pgid/@selected");
	vopt[PPID] = getNodeIntVal("/configuration/viewoptions/ppid/@selected");
	vopt[USER] = getNodeIntVal("/configuration/viewoptions/user/@selected");
	vopt[FD] = getNodeIntVal("/configuration/viewoptions/fd/@selected");
	vopt[TYPE] = getNodeIntVal("/configuration/viewoptions/type/@selected");
	vopt[DEVICE] = getNodeIntVal("/configuration/viewoptions/device/@selected");
	vopt[SIZEOFFSET] = getNodeIntVal("/configuration/viewoptions/sizeoffset/@selected");
	vopt[NLINK] = getNodeIntVal("/configuration/viewoptions/nlink/@selected");
	vopt[NODE] = getNodeIntVal("/configuration/viewoptions/node/@selected");
}

void setViewOptions(unsigned int vopt[]) {
	char str[2];

	sprintf(str, "%d", vopt[PID]);
	setNodeContent("/configuration/viewoptions/pid/@selected", str);
	sprintf(str, "%d", vopt[PGID]);
	setNodeContent("/configuration/viewoptions/pgid/@selected", str);
	sprintf(str, "%d", vopt[PPID]);
	setNodeContent("/configuration/viewoptions/ppid/@selected", str);
	sprintf(str, "%d", vopt[USER]);
	setNodeContent("/configuration/viewoptions/user/@selected", str);
	sprintf(str, "%d", vopt[FD]);
	setNodeContent("/configuration/viewoptions/fd/@selected", str);
	sprintf(str, "%d", vopt[TYPE]);
	setNodeContent("/configuration/viewoptions/type/@selected", str);
	sprintf(str, "%d", vopt[DEVICE]);
	setNodeContent("/configuration/viewoptions/device/@selected", str);
	sprintf(str, "%d", vopt[SIZEOFFSET]);
	setNodeContent("/configuration/viewoptions/sizeoffset/@selected", str);
	sprintf(str, "%d", vopt[NLINK]);
	setNodeContent("/configuration/viewoptions/nlink/@selected", str);
	sprintf(str, "%d", vopt[NODE]);
	setNodeContent("/configuration/viewoptions/node/@selected", str);
}


/* lsofsettings */
/* general*/
unsigned int getAndsettings() {
	return getNodeIntVal("/configuration/lsofsettings/general/andsettings/@selected");
}

void setAndsettings(unsigned int val) {
	char str[2];

	sprintf(str, "%d", val);
	setNodeContent("/configuration/lsofsettings/general/andsettings/@selected", str);
}


unsigned int getIdorlogin() {
	xmlChar *ret = getNodeStringVal("/configuration/lsofsettings/general/idorlogin");

	if(0 == strcmp(ret, "id"))
		return 0;
	else if(0 == strcmp(ret, "login"))
		return 1;
	else
		return ERRCONF;
}

void setIdorlogin(unsigned int val) {
	if(0 == val)
		setNodeContent("/configuration/lsofsettings/general/idorlogin", "id");
	else
		setNodeContent("/configuration/lsofsettings/general/idorlogin", "login");
}


unsigned int getOffsetorsize() {
	xmlChar *tmp = getNodeStringVal("/configuration/lsofsettings/general/offsetorsize");

	if(0 == strcmp(tmp, "offset"))
		return 0;
	else if(0 == strcmp(tmp, "size"))
		return 1;
	else
		return ERRCONF;
}

void setOffsetorsize(unsigned int val) {
	if(0 == val)
		setNodeContent("/configuration/lsofsettings/general/offsetorsize", "offset");
	else
		setNodeContent("/configuration/lsofsettings/general/offsetorsize", "size");
}


unsigned int getHidefromoutput() {
	return getNodeIntVal("/configuration/lsofsettings/general/hidefromoutput/@selected");
}

void setHidefromoutput(unsigned int val) {
	char str[2];

	sprintf(str, "%d", val);
	setNodeContent("/configuration/lsofsettings/general/hidefromoutput/@selected", str);
}


/* files */
unsigned int getProcessesexecuting(xmlChar **str) {
	*str = getNodeStringVal("/configuration/lsofsettings/files/processesexecuting");
	return getNodeIntVal("/configuration/lsofsettings/files/processesexecuting/@selected");
	
}

void setProcessesexecuting(unsigned int val, xmlChar *str) {
	char tmp[2];

	setNodeContent("/configuration/lsofsettings/files/processesexecuting", str);

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/files/processesexecuting/@selected", tmp);
	
}


unsigned int getIdprocesses(xmlChar **str) {
	*str = getNodeStringVal("/configuration/lsofsettings/files/idprocesses");
	return getNodeIntVal("/configuration/lsofsettings/files/idprocesses/@selected");

}

void setIdprocesses(unsigned int val, xmlChar *str) {
	char tmp[2];

	setNodeContent("/configuration/lsofsettings/files/idprocesses", str);

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/files/idprocesses/@selected", tmp);
}


unsigned int getPgidprocesses(xmlChar **str) {
	*str = getNodeStringVal("/configuration/lsofsettings/files/pgidprocesses");
	return getNodeIntVal("/configuration/lsofsettings/files/pgidprocesses/@selected");
}

void setPgidprocesses(unsigned int val, xmlChar *str) {
	char tmp[2];

	setNodeContent("/configuration/lsofsettings/files/pgidprocesses", str);

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/files/pgidprocesses/@selected", tmp);
}


/* processes */
unsigned int getDisplaydirectory(xmlChar **str, unsigned int *depth) {
	*depth = getNodeIntVal("/configuration/lsofsettings/processes/displaydirectory/@completedepth");

	*str = getNodeStringVal("/configuration/lsofsettings/processes/displaydirectory");

	return getNodeIntVal("/configuration/lsofsettings/processes/displaydirectory/@selected");
}

void setDisplaydirectory(unsigned int val, xmlChar *str, unsigned int depth) {
	char tmp[2];

	setNodeContent("/configuration/lsofsettings/processes/displaydirectory", str);

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/processes/displaydirectory/@selected", tmp);

	sprintf(tmp, "%d", depth);
	setNodeContent("/configuration/lsofsettings/processes/displaydirectory/@completedepth", tmp);
}


unsigned int getFiledescriptors(xmlChar **str, unsigned int *act) {
	xmlChar *tmp = getNodeStringVal("/configuration/lsofsettings/processes/filedescriptors/@action");
	if(0 == strcmp("include", tmp))
		*act = 0;
	else if(0 == strcmp("exclude", tmp))
		*act = 1;
	else {
		*act = ERRCONF;
		return ERRCONF;
	}
	
	*str = getNodeStringVal("/configuration/lsofsettings/processes/filedescriptors");
	return getNodeIntVal("/configuration/lsofsettings/processes/filedescriptors/@selected");
}

void setFiledescriptors(unsigned int val, xmlChar *str, unsigned int act) {
	char tmp[2];

	if(0 == act)
		setNodeContent("/configuration/lsofsettings/processes/filedescriptors/@action", "include");
	else
		setNodeContent("/configuration/lsofsettings/processes/filedescriptors/@action", "exclude");

	setNodeContent("/configuration/lsofsettings/processes/filedescriptors", str);

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/processes/filedescriptors/@selected", tmp);
}


unsigned int getNameandid(xmlChar **str) {
	*str = getNodeStringVal("/configuration/lsofsettings/processes/nameandid");
	return getNodeIntVal("/configuration/lsofsettings/processes/nameandid/@selected");
}

void setNameandid(unsigned int val, xmlChar *str) {
	char tmp[2];

	setNodeContent("/configuration/lsofsettings/processes/nameandid", str);

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/processes/nameandid/@selected", tmp);
}


unsigned int getMaxlinks(xmlChar **num) {
	*num = getNodeStringVal("/configuration/lsofsettings/processes/maxlinks");
	return getNodeIntVal("/configuration/lsofsettings/processes/maxlinks/@selected");
}

void setMaxlinks(unsigned int val, xmlChar *max) {
	char tmp[2];

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/processes/maxlinks/@selected", tmp);

	setNodeContent("/configuration/lsofsettings/processes/maxlinks", max);
}


/* network */
unsigned int getInternetaddressesSelected() {
	return getNodeIntVal("/configuration/lsofsettings/network/internetaddresses/@selected");
}

unsigned int getInternetaddresses(GPtrArray *tmp_list) {
	int n, i;
	char *path;

	// get internetaddress nodes
	xmlNodeSetPtr childs = getNodes("/configuration/lsofsettings/network/internetaddresses/internetaddress");
	if(NULL == childs || 0 == childs->nodeNr) {
                return 0;
        }
	
	// # of internetaddresses childs
	n = childs->nodeNr;

	for(i = 0; i < n; i++) {
		//internetAddress *tmp_ia = malloc(sizeof(internetAddress));
		node_data *tmp_ia = g_new0 (node_data, 1);
		xmlNodePtr child = childs->nodeTab[i];
		path = xmlGetNodePath(child);
		// 10 is the length of "/protocol\0"
		path = calloc(sizeof(char), strlen(path) + 10);

		path = xmlGetNodePath(child);
		path = strcat(path, "/host");
		tmp_ia->host = getNodeStringVal(path);

		path = xmlGetNodePath(child);
		path = strcat(path, "/port");
		tmp_ia->port = getNodeStringVal(path);

		path = xmlGetNodePath(child);
		path = strcat(path, "/ipv");
		tmp_ia->ipv = getNodeStringVal(path);

		path = xmlGetNodePath(child);
		path = strcat(path, "/protocol");
		tmp_ia->protocol = getNodeStringVal(path);
		
		/* make option string */
		
		tmp_ia->option = g_strconcat (tmp_ia->ipv, tmp_ia->protocol, NULL);
		
		if (strlen (tmp_ia->host))
			tmp_ia->option = g_strconcat (tmp_ia->option, "@", tmp_ia->host, NULL);
		
		if (strlen (tmp_ia->port))
			tmp_ia->option = g_strconcat (tmp_ia->option, ":", tmp_ia->port, NULL);

		if (strlen(tmp_ia->option))
			g_ptr_array_add (tmp_list, (gpointer) tmp_ia);
			
		/* _make option string */
		//internetAddresses[i] = tmp_ia;
		free(path);
	}

	return n;
}


void setInternetaddressesSelected(unsigned int val) {
	char tmp[2];

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/network/internetaddresses/@selected", tmp);
}


void setInternetaddresses(GPtrArray *tmp_list) {
	int i;
        xmlNodePtr parent;
        char *parentpath = "/configuration/lsofsettings/network/internetaddresses";
	node_data * tmp_ia;

        /* delete all child nodes */
        setNodeContent(parentpath, "");

        /* extract internetaddresses node */
        xmlXPathObjectPtr ptr = xmlXPathEval(parentpath, xpcont);
        parent = ptr->nodesetval->nodeTab[0];

        for(i = 0; i < tmp_list->len; i++) {
                tmp_ia = g_ptr_array_index (tmp_list, i);

                xmlNodePtr ia = xmlNewChild(parent, NULL, "internetaddress", NULL);

                xmlNewChild(ia, NULL, "host", tmp_ia->host);
                xmlNewChild(ia, NULL, "port", tmp_ia->port);
                xmlNewChild(ia, NULL, "ipv", tmp_ia->ipv);
                xmlNewChild(ia, NULL, "protocol", tmp_ia->protocol);
        }
}

unsigned int getInternetaddress(xmlChar **str) {
	*str = getNodeStringVal("/configuration/lsofsettings/network/internetaddress");
	return getNodeIntVal("/configuration/lsofsettings/network/internetaddress/@selected");
}

void setInternetaddress(unsigned int val, xmlChar *str) {
	char tmp[2];

	setNodeContent("/configuration/lsofsettings/network/internetaddress", str);

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/network/internetaddress/@selected", tmp);
}


unsigned int getIpaddress() {
	return getNodeIntVal("/configuration/lsofsettings/network/ipaddress/@selected");
}

void setIpaddress(unsigned int val) {
	char tmp[2];

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/network/ipaddress/@selected", tmp);
}


unsigned int getPortnumber() {
	return getNodeIntVal("/configuration/lsofsettings/network/portnumber/@selected");
}

void setPortnumber(unsigned int val) {
	char tmp[2];

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/network/portnumber/@selected", tmp);
}


unsigned int getNfsfiles() {
	return getNodeIntVal("/configuration/lsofsettings/network/nfsfiles/@selected");
}

void setNfsfiles(unsigned int val) {
	char tmp[2];

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/network/nfsfiles/@selected", tmp);
}


unsigned int getUnixsocket() {
	return getNodeIntVal("/configuration/lsofsettings/network/unixsocket/@selected");
}

void setUnixsocket(unsigned int val) {
	char tmp[2];

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/network/unixsocket/@selected", tmp);
}



/* kernel */
unsigned int getTimeout(xmlChar **num) {
	*num = getNodeStringVal("/configuration/lsofsettings/kernel/timeout");
	return getNodeIntVal("/configuration/lsofsettings/kernel/timeout/@selected");
}

void setTimeout(unsigned int val, xmlChar *time) {
	char tmp[2];

	setNodeContent("/configuration/lsofsettings/kernel/timeout", time);

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/kernel/timeout/@selected", tmp);
}


unsigned int getAvoiddangerous() {
	return getNodeIntVal("/configuration/lsofsettings/kernel/avoiddangerous/@selected");
}

void setAvoiddangerous(unsigned int val) {
	char tmp[2];

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/lsofsettings/kernel/avoiddangerous/@selected", tmp);
}




/* gtklsof config */
/* lsof path */
void getLsofpath(xmlChar **str) {
	*str = getNodeStringVal("/configuration/config/lsofpath");
}

void setLsofpath(xmlChar *str) {
	setNodeContent("/configuration/config/lsofpath", str);
}

/* size and position */
void getSizePosition(unsigned int sp[]) {
	sp[X] = getNodeIntVal("/configuration/config/keepsizeposition/x");
	sp[Y] = getNodeIntVal("/configuration/config/keepsizeposition/y");
	sp[WIDTH] = getNodeIntVal("/configuration/config/keepsizeposition/width");
	sp[HEIGHT] = getNodeIntVal("/configuration/config/keepsizeposition/height");
}

void setSizePosition(unsigned int sp[]) {
	char tmp[2];

	sprintf(tmp, "%d", sp[X]);
	setNodeContent("/configuration/config/keepsizeposition/x", tmp);

	sprintf(tmp, "%d", sp[Y]);
	setNodeContent("/configuration/config/keepsizeposition/y", tmp);

	sprintf(tmp, "%d", sp[WIDTH]);
	setNodeContent("/configuration/config/keepsizeposition/width", tmp);

	sprintf(tmp, "%d", sp[HEIGHT]);
	setNodeContent("/configuration/config/keepsizeposition/height", tmp);
}

unsigned int getKeepsizeposition() {
	return getNodeIntVal("/configuration/config/keepsizeposition/@selected");
}

void setKeepsizeposition(unsigned int val) {
	char tmp[2];

	sprintf(tmp, "%d", val);
	setNodeContent("/configuration/config/keepsizeposition/@selected", tmp);
}

/* show/hide splash window */
unsigned int getSplashWindow() {
	                return getNodeIntVal("/configuration/config/hidesplashwindow/@selected");
}

void setSplashWindow(unsigned int val) {
	                char tmp[2];

			sprintf(tmp, "%d", val);
			setNodeContent("/configuration/config/hidesplashwindow/@selected", tmp);
}



/* utility functions */
unsigned int getNodeIntVal(xmlChar *path) {
	xmlXPathObjectPtr obj = xmlXPathEval(path, xpcont);

	return (unsigned int)xmlXPathCastToNumber(obj);
}

xmlChar *getNodeStringVal(xmlChar *path) {
	xmlXPathObjectPtr obj = xmlXPathEval(path, xpcont);

	return xmlXPathCastToString(obj);
}

xmlNodeSetPtr getNodes(xmlChar *path) {
	xmlXPathObjectPtr ptr = xmlXPathEval(path, xpcont);
        if(ptr != NULL) {
                return ptr->nodesetval;
        }
        return NULL;
}

void setNodeContent(xmlChar *path, xmlChar *content) {
	xmlNodePtr node;
        xmlXPathObjectPtr ptr = xmlXPathEval(path, xpcont);
        node = ptr->nodesetval->nodeTab[0];
        xmlNodeSetContent(node, content);
}



/* check configuration file */
unsigned int checkConfig(char *filename) {
	xmlParserCtxtPtr ptxt;
	xmlValidCtxtPtr ctxt;
	xmlDtdPtr dtd;

	doc = xmlParseFile(filename);
	if(NULL == doc) {
		cleanup();
		return -1;
	}

	ptxt = xmlCreateDocParserCtxt(filename);
	if(NULL == ptxt) {
		cleanup();
		return -1;
	}

	ctxt = &(ptxt->vctxt);
	if(NULL == ctxt) {
		xmlFreeParserCtxt(ptxt);
		ptxt = NULL;
		cleanup();
		return -1;
	}


	dtd = xmlParseDTD(NULL, DTD);
	if(NULL == dtd) {
		xmlFreeParserCtxt(ptxt);
		ptxt = NULL;
		cleanup();
		return -1;
	}

	if(0 == xmlValidateDtd(ctxt, doc, dtd)) {
		xmlFreeParserCtxt(ptxt);
		ptxt = NULL;
		cleanup();
		return ERRCONF;
	}

	xmlFreeParserCtxt(ptxt);
	ptxt = NULL;
	cleanup();
	return 0;
}

void cleanup() {
	if(NULL != doc) xmlFreeDoc(doc);
	doc = NULL;
	if(NULL != xpcont) xmlXPathFreeContext(xpcont);
	xpcont = NULL;
	xmlCleanupParser();
}
